package debug;

use strict;
use warnings;

use Quak::Plugins;
use Quak::Utilities;

my $debug = {
	'auth' => 0,
	'message' => 0,
	'iq' => 0,
	'presence' => 0,
	'send' => 0,
	'process' => 0
};

Quak::Plugins::register(
	['auth', \&onauth, undef],
	['message', \&onmessage, qr/./],
	['iq', \&oniq, undef],
	['presence', \&onpresence, undef],
	['send', \&onsend, undef],
#	['process', \&onprocess, undef],
);

sub onsend {
	return unless $debug->{'send'};
	printf "*** in %s: sending XML\n%s\n", (caller(0))[3], $_[0]
}

sub onauth {
	return unless $debug->{'auth'};
	printf "in %s\n", (caller(0))[3];
}

sub onmessage {
	if ($_[0]->GetBody =~ /^debug\s+(auth|message|iq|presence|send|process|all)\s+(on|off)/) {
		if ($1 eq 'all') {
			foreach my $d (keys %{$debug}) {$debug->{$d} = ($2 eq 'on')?1:0}
			return
		}
		$debug->{$1} = ($2 eq 'on')?1:0;
		return
	}
	if ($_[0]->GetBody =~ /^debug$/) {
		my $reply = $_[0]->Reply;
		my $body = "\n";
		foreach my $d (keys %{$debug}) {$body .= sprintf "debug %s %d\n", $d, $debug->{$d}}
		$reply->SetMessage('body' => $body);
		Send($reply);
		return
	}
	return unless $debug->{'message'};
	printf "*** in %s: message received.\n", (caller(0))[3];
	printf "\tfrom:    [%s]\n", $_[0]->GetFrom,
	printf "\tto:      [%s]\n", $_[0]->GetTo,
	printf "\tsubject: [%s]\n", $_[0]->GetSubject,
	printf "\tbody:    [%s]\n", $_[0]->GetBody;
}

sub oniq {
	return unless $debug->{'iq'};
	printf "*** in %s: iq received.\n", (caller(0))[3];
	printf "\tfrom:  [%s]\n", $_[0]->GetFrom;
	printf "\tto:    [%s]\n", $_[0]->GetTo;
	printf "\ttype:  [%s]\n", $_[0]->GetType;
	printf "\tid:    [%s]\n", $_[0]->GetID;
	printf "\tquery: [%s]\n", $_[0]->GetQuery->GetXMLNS;
	printf "\txml:   [\n%s\n]\n", $_[0]->GetXML;
}

sub onpresence {
	return unless $debug->{'presence'};
	printf "*** in %s: presence received.\n", (caller(0))[3];
	printf "\tfrom:   [%s]\n", ($_[0]->GetFrom or '');
	printf "\tto:     [%s]\n", ($_[0]->GetTo or '');
	printf "\ttype:   [%s]\n", ($_[0]->GetType or '');
	printf "\tstatus: [%s]\n", ($_[0]->GetStatus or '');
}

sub onprocess {
	return unless $debug->{'process'};
	printf "*** in %s\n", (caller(0))[3];
}
1
