package Quak::Plugins;

require Exporter;

use strict;
use warnings;

our @ISA = qw/Exporter/;
our @EXPORT = qw/register loadPlugins getPlugins/;

my $plugins;

sub register {
	foreach my $d (@_) {
		push @{$plugins->{$d->[0]}}, {func => $d->[1], args => $d->[2]}
	}
	return 1
}

#sub loadAllPlugins {
#	foreach (glob "Plugins/*.pm") {
#		s!/!::!g;
#		require $_
#	}
#}

sub loadPlugins {
	return unless defined $::cfg->{'plugindir'};
	return unless -d $::cfg->{'plugindir'};
	foreach (split /[ ,]+/, $::cfg->{'plugins'}) {
		eval { require "$::cfg->{'plugindir'}/$_.pm" };
		if ($@) {printf "Error in $_: $@\n"}
	}
}

sub getPlugins {
	return $plugins->{$_[0]}||undef
}

1;

__END__

=head1 NAME

Quak::Plugins - Required to write plugins for the Quack Jabber Component

=head1 SYNOPSIS

 use Quak::Plugins

 Quak::Plugins::register([$hook, \&handler, $message]);
 Quak::Plugins::register([$hook', sub {..}, qr/regex/]);

=head1 DESCRIPTION

With Quak::Plugins you can hook to several Jabber events and make the component
react as desired.

=head1 METHODS

=over 8

=item B<Register>(I<$hook>, I<\&handler>, [I<$message>], ..);

Register plugin: hook to I<$hook> and call I<handler()> if the corresponding
event was received (and I<$message> matches, where applicable [B<message> only]).

=back

=head1 HOOKS

=over 8

=item B<send>

=item B<message>

=item B<presence>

=item B<iq>

=item B<onconnect>

=item B<onauth>

=item B<onprocess>

=item B<onauthfail>

=item B<ondisconnect>

=item B<onexit>

=back

=head1 EXAMPLES

 package echo;

 use Quak::Plugins;
 use Quak::Utilities;

 Quak::Plugins::register(
     ['message', \&msgHandler, qr/^hello\s*/]
  );

 sub msgHandler {
     my $reply = $_[0]->Reply;
     $reply->SetMessage('body' => 'got this: '. $_[0]->GetBody);
     Send($reply);
 }

 1

=head1 SEE ALSO

L<Quak::Utilities>

=head1 AUTHOR

Philippe Kueck <projects at unixadm dot org>

=cut
