package Quak::Utilities;

use strict;
use warnings;
our @ISA = qw/Exporter/;
our @EXPORT = qw/Send SendMessage SendChat db_err/;

sub db_err {
	printf "Database error: %s\n", $DBI::errstr;
	return 0
}

sub Send {
	$::c->Send($_[0])
}

sub SendChat {
	# $_[1] is ignored for backwards compatibility
	my $msg = new Net::Jabber::Message;
	$msg->SetMessage(
		'to' => $_[0],
		'from' => $::cfg->{'me'},
		'type' => 'chat',
		'body' => $_[2]
	);
	&Send($msg)
}

sub SendMessage {
	my $msg = new Net::Jabber::Message;
	$msg->SetMessage(
		'to' => $_[0],
		'from' => $::cfg->{'me'},
		'type' => 'normal',
		'subject' => $_[1],
		'body' => $_[2]
	);
	&Send($msg)
}

1;

__END__

=head1 NAME

Quak::Utilities - Simple event sending wrappers

=head1 SYNOPSIS

 use Quak::Utilities;
 SendMessage(I<recipient>, I<subject>, I<body>, ['chat']);
 SendChat(I<recipient>, I<subject>, I<body>);
 Send(I<Net::Jabber::Message>);

=head1 DESCRIPTION

 Quak::Utilities can be used to make quak send messages.

=head1 METHODS

=over 8

=item B<SendMessage>(I<recipient>, I<subject>, I<body>, [B<'chat'>]);

Sends a message (or a B<chat>) to the given I<recipient>.

=item B<SendChat>(I<recipient>, I<subject>, I<body>);

Sends a chat to the given I<recipient>.

=item B<Send>(I<Net::Jabber::Message>);

Sends a raw Jabber Message.

=back

=head1 EXAMPLES

 use Quak::Utilities;
 my $msg = new Net::Jabber::Message;
 ...
 Send($msg);

 SendMessage('user@jabber.example', 'my subject', 'Hi there');

=head1 SEE ALSO

L<Quak::Plugins>

=head1 AUTHOR

Philippe Kueck <projects at unixadm dot org>

=cut
