package Quak::XEP0133;

use strict;
use warnings;

use Quak::Roster;
use Quak::Utilities;
use Quak::NamespacesCopy;

our @ISA = qw/Exporter/;
our @EXPORT = qw/admin/;

###
# admin command sending method
###

sub admin {
	my $iq = new Net::XMPP::IQ;
	$iq->SetIQ(
		'from' => $::cfg->{'me'},
		'type' => 'set',
		'to' => $::cfg->{'domain'}
	);
	my $cmd = $iq->NewChild('http://jabber.org/protocol/commands');
	$cmd->SetAction("execute");
	$cmd->SetNode("http://jabber.org/protocol/admin#$_[0]");
	Send($iq)
}

sub subscribe_to_presence {
	my $reply = $_[0]->Reply;
	$reply->SetType("set");
	$reply->GetQuery->SetNode($_[0]->GetQuery->GetNode);
	$reply->GetQuery->SetSessionID($_[0]->GetQuery->GetSessionID);
	my $x = $reply->GetQuery->NewChild("jabber:x:data");
	$x->SetType("submit");
	my $f_hidden = $x->AddField;
	$f_hidden->SetType("hidden");
	$f_hidden->SetVar("FORM_TYPE");
	$f_hidden->SetValue("http://jabber.org/protocol/admin");
	my $f_packet_type = $x->AddField;
		$f_packet_type->SetVar("packet_type");
		$f_packet_type->SetValue(["presence"]);
	my $f_direction = $x->AddField;
		$f_direction->SetVar("direction");
		$f_direction->SetValue("incoming");
	my $f_processed = $x->AddField;
		$f_processed->SetVar("processed");
		$f_processed->SetValue("false");
	Send($reply)
}

sub executing_list_form {
	my $reply = $_[0]->Reply;
	$reply->SetType("set");
	$reply->GetQuery->SetNode($_[0]->GetQuery->GetNode);
	$reply->GetQuery->SetSessionID($_[0]->GetQuery->GetSessionID);
	my $x = $reply->GetQuery->NewChild("jabber:x:data");
	$x->SetType("submit");
	my $f_hidden = $x->AddField;
	$f_hidden->SetType("hidden");
	$f_hidden->SetVar("FORM_TYPE");
	$f_hidden->SetValue("http://jabber.org/protocol/admin");
	my $f_data = $x->AddField;
	$f_data->SetVar("max_items");
	$f_data->SetValue("none");
	Send($reply)
}

sub completed_get_active_users {
	my $x = $_[0]->GetQuery->GetX;
	my $users;
	foreach ($x->GetFields) {
		next unless $_->GetVar eq 'activeuserjids';
		next if $_->GetType eq 'hidden';
		push @$users, $_->GetValue
	}
	$::roster->refresh_online($users)
}


sub dumpraw {
	my $x = $_[0]->GetQuery->GetX;
	foreach ($x->GetFields) {
		next if $_->GetType eq 'hidden';
		printf "%s {\n", $_->GetVar;
		foreach my $v ($_->GetValue) {printf "\t%s\n\n", $v}
		printf "}\n"
	}
}

# fwiw
sub completed_get_active_presences {
	dumpraw $_[0];
}

sub completed_get_server_stats {
	my $x = $_[0]->GetQuery->GetX;
	foreach ($x->GetFields) {
		next if $_->GetType eq 'hidden';
		$::stats->{$_->GetVar} = $_->GetValue
	}
}

###
# Handler
###

sub command_handler {
	my $q = $_[0]->GetQuery;
	if ($q->GetNode =~ m!^http://jabber\.org/protocol/admin#(.+)$!) {
		if ($1 eq 'packets_notification') {
			if ($q->GetStatus eq 'executing') {&subscribe_to_presence}
			return
		}
		if ($q->GetStatus eq 'executing') {&executing_list_form($_[0])}
		elsif ($q->GetStatus eq 'completed') {
			my $func = "completed_$1";
			$func =~ s/-/_/g;
			&{\&$func}($_[0])
		}
	}
}

sub connectHandler {
	my $presence = new Net::XMPP::Presence;
	my @online = $::roster->subscribed_jids;
	foreach (@online) {
		$presence->SetPresence('to' => $_, 'from' => $::cfg->{'me'}, 'type' => 'available');
		Send($presence)
	}
	&admin("get-active-users")
	&admin("packets_notification")
}

sub disconnectHandler {
	my $presence = new Net::XMPP::Presence;
	my @online = $::roster->online_subscribed_jids;
	foreach (@online) {
		$presence->SetPresence('to' => $_, 'from' => $::cfg->{'me'}, 'type' => 'unavailable');
		Send($presence)
	}
}

sub copyHandler {
	return unless $_[0]->DefinedChild;
	my $copy = $_[0]->GetChild;
	if ($copy->GetXMLNS eq 'http://jabber.org/protocol/packet#event') {
		if ($copy->DefinedPresence && $copy->GetIncoming eq "true") {
			# ignore everything but messages _from_ users _to_ server
			# e.g. From _must_ be defined, To _must not_ be defined
			return unless $copy->DefinedPresenceFrom;
			return if $copy->DefinedPresenceTo;
			$::roster->online($copy->GetPresenceFrom, 1) if !$copy->DefinedPresenceType || $copy->GetPresenceType eq "probe";
			$::roster->online($copy->GetPresenceFrom, 0) if $copy->GetPresenceType eq "unavailable";
#			my $presence = new Net::XMPP::Presence;
#			$presence->SetFrom($copy->GetPresenceFrom);
#			$presence->SetTo($copy->GetPresenceTo);
#			$presence->SetType($copy->GetPresenceType);
#			::presenceCB(undef, $presence)			
		}
	}
}

my $timeout = {
	'time' => time,
	'timeout' => 30
};

sub update_get_active_users {
	return unless $timeout->{'time'} + $timeout->{'timeout'} < time;
	$timeout->{'time'} = time;
	&admin("get-active-users")
}

1
